import re
from .interfaces import IFanstaticSet
from .apis import FanstaticSet

def add_fanstatic_resources(config, resources, renderer_name_pattern):
    """ Add fanstatic resources for renderers matched to `renderer_name_pattern`.

    Don't use directly this function. 
    You should use that as directive after include "rebecca.fanstatic".
    """

    fanstatic_set = config.registry.queryUtility(IFanstaticSet)
    if fanstatic_set is None:
        fanstatic_set = FanstaticSet()
        config.registry.registerUtility(fanstatic_set, IFanstaticSet)
    renderer_name_regex = re.compile(renderer_name_pattern)
    resources = [config.maybe_dotted(r) for r in resources]
    
    fanstatic_set.add_fanstatic(resources, renderer_name_regex)



import os
import shutil
import argparse
from fanstatic.registry import get_library_registry

def _print(s):
    print s


def iter_all_libraries():
    reg = get_library_registry()
    return reg.itervalues()

def iter_libraries(names):
    reg = get_library_registry()
    return (v for v in reg.itervalues() if v.name in names)

def print_fanstatic_resources(destbasedir, library):
    outdir = os.path.join(destbasedir, library.name)
    for name, resource in library.known_resources.iteritems():
        sourcepath = os.path.join(library.path, resource.relpath)
        dest = os.path.join(outdir, resource.relpath)
        _print("%s -> %s" % (sourcepath, dest))

def copy_fanstatic_resources(destbasedir, library):
    outdir = os.path.join(destbasedir, library.name)

    if not os.path.exists(outdir):
        os.mkdir(outdir)

    for name, resource in library.known_resources.iteritems():
        sourcepath = os.path.join(library.path, resource.relpath)
        dest = os.path.join(outdir, resource.relpath)
        destdirname = os.path.dirname(dest)
        if not os.path.exists(destdirname):
            os.mkdir(destdirname)
        _print("copy %s -> %s" % (sourcepath, dest))
        shutil.copyfile(sourcepath, dest)

def list_fanstatic():
    parser = argparse.ArgumentParser(description="Collect fanstatic resources and copy to destination directory.")
    parser.add_argument('-o', '--output-directory', default="fanstatic", 
        help="Set destination directory name. Default directory name is 'fanstatic'.")
    parser.add_argument('-l', '--library', nargs="*", 
        help="Specify copy library names. Accept multiple options.")
    parser.add_argument('--simulate', action='store_true', 
        help="Simulate collecting resources. Print resource paths and destination paths.")

    args = parser.parse_args()
    if not args.simulate and not os.path.exists(args.output_directory):
        os.mkdir(args.output_directory)

    destbasedir = args.output_directory
    if not args.library:
        lib_iter = iter_all_libraries()
    else:
        lib_iter = iter_libraries(args.library)

    for library in lib_iter:
        if args.simulate:
            print_fanstatic_resources(destbasedir, library)
        else:
            copy_fanstatic_resources(destbasedir, library)




import unittest
import webtest
from pyramid import testing

class FunctionalTests(unittest.TestCase):

    def setUp(self):
        self.config = testing.setUp()
        from fanstatic.core import init_needed
        init_needed()

    def tearDown(self):
        from fanstatic.core import del_needed
        del_needed()
        testing.tearDown()


    def test_directive(self):
        self.config.include('rebecca.fanstatic')
        self.assertTrue(hasattr(self.config, 'add_fanstatic_resources'))

    def test_notify(self):
        from pyramid.events import BeforeRender
        from fanstatic.core import get_needed


        self.config.include('rebecca.fanstatic')
        self.config.add_fanstatic_resources(['js.jquery.jquery',], r'.*\.mak')

        request = testing.DummyRequest(registry=self.config.registry)
        system_values = {'request': request, 'renderer_name': 'test.mak'}
        event = BeforeRender(system_values)

        self.config.registry.notify(event)

        from js.jquery import jquery
        needed = get_needed()
        self.assertIn(jquery, needed._resources)




from zope.interface import Interface

class IFanstaticSet(Interface):

    def add_fanstatic(resources, renderer_name_regex):
        """ add fanstatic resources needed by renderer matches renderer_name_pattern"""


    def __call__(renderer_name):
        """ invoke `Resource.need` trigger of matched resources"""



#

def includeme(config):
    config.add_directive('add_fanstatic_resources', '.directives.add_fanstatic_resources')
    config.add_subscriber('.subscribers.need_fanstatic', 'pyramid.events.BeforeRender')




from fanstatic import get_library_registry
from zope.interface import implementer
from .interfaces import IFanstaticSet

def get_fanstatic_set(request):
    reg = request.registry
    fanstatic_set = reg.queryUtility(IFanstaticSet)
    return fanstatic_set

def need_fanstatic(request, renderer_name):
    fanstatic_set = get_fanstatic_set(request)
    if fanstatic_set is None:
        return
    fanstatic_set(renderer_name)

class FanstaticSet(object):
    def __init__(self):
        self.fanstatics = []

    def add_fanstatic(self, resources, renderer_name_pattern):
        self.fanstatics.append(Fanstatic(resources, renderer_name_pattern))

    def __call__(self, renderer_name):

        for f in self.fanstatics:
            f(renderer_name)

class Fanstatic(object):
    def __init__(self, resources, renderer_name_regex):
        self.resources = resources
        self.regex = renderer_name_regex

    def match(self, renderer_name):
        return self.regex.match(renderer_name)

    def __call__(self, renderer_name):
        if self.match(renderer_name):
            for resource in self.resources:
                from fanstatic.core import get_needed
                resource.need()



from fanstatic import Library, Resource

dummy_library = Library('rebecca.fanstatic.dummy_library', 'dummy_resources')
dummy1_js = Resource(dummy_library, 'dummy1.js')
dummy2_js = Resource(dummy_library, 'dummy2.js')
dummy_css = Resource(dummy_library, 'dummy.css')



from setuptools import setup, find_packages
import os

__version__ = '0.1.2'
__author__ = 'Atsushi Odagiri'

requires = [
    "fanstatic",
]

testing_requires = [
    "nose",
    "coverage",
    "mock",
    "webtest",
]

try:
    import argparse
except ImportError:
    requires.append('argparse')

points = {
    "console_scripts": [
        "list_fanstatic=rebecca.fanstatic.commands:list_fanstatic",
    ],
    "fanstatic.libraries": [
        "rebecca.fanstatic.dummy_library=rebecca.fanstatic.tests.dummylibrary:dummy_library",
    ],
}
here = os.path.dirname(__file__)

def _read(name):
    try:
        return open(os.path.join(here, name)).read()
    except:
        return ""

        
readme = _read("README.txt")
changes = _read("CHANGES.txt")

setup(name="rebecca.fanstatic",
    version=__version__,
    author=__author__,
    author_email="aodagx@gmail.com",
    url="http://github.com/rebeccaframework/rebecca.fanstatic",
    packages=find_packages(),
    namespace_packages=['rebecca'],

    install_requires=requires,
    entry_points=points,
    extras_require= {
        "testing": testing_requires,
    },
    description="fanstatic utilities",
    long_description=readme+"\r\n"+changes,
    license="MIT",
    package_data={
        'rebecca.fanstatic.tests': ['dummy_resources/*.js', 'dummy_resources/*.css'],
    },
)

